/*Written by S. Li and N. Sandford for Pomona College. Some code has be taken from Adafruit and Arduino’s tutorials. May, 2015
*/

// include the library code:
#include <Wire.h>
#include <Adafruit_MCP23017.h>
#include <Adafruit_RGBLCDShield.h>
#include <EEPROM.h>

// The shield uses the I2C SCL and SDA pins. On classic Arduinos
// this is Analog 4 and 5 so you can't use those for analogRead() anymore
// However, you can connect other I2C sensors to the I2C bus and share
// the I2C bus.
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// These #defines make it easy to set the backlight color
#define RED 0x1
#define YELLOW 0x3
#define GREEN 0x2
#define TEAL 0x6
#define BLUE 0x4
#define VIOLET 0x5
#define WHITE 0x7


int ptr = 0;  // menu pointer
int motor = A0; // motor/generator
int batt = A1; // 12V battery
int hall = A2; // for current sensing
int pdiode = A3; // photodiode for rpm
int mosfet = 7; // used to turn on the inverter
int cc = 8; //charge controller
int photodiode = 470; //random initialization
int cycle = 0 ; // t0, cycle used in rpm calculation
int numCycle = 20; // for averaging use
int mins, secs, kmh;
float Wh = 0; //watthours
int wait = 500; //delay in milliseconds
unsigned long timeOn, timecheck; // for time checking
double revolution = 0.00175929; //kilometer
//long term storage variables
int timeAddress = 0;
int distanceAddress = 1;
int powerAddress = 2;
byte timeValue, distanceValue, powerValue;

float afactor = 1023.0; //analog read max
float motorV, battV, rpm, t1, current;
float t0 = 0.0;
boolean hasBeenOn = false;
boolean debugging = true;
boolean mosfetOn = false;
boolean isHome = true;

void setup() {
  // put your setup code here, to run once:
  lcd.begin(16, 2);
  lcd.print("Hello, world!");
  lcd.setBacklight(WHITE);
  pinMode(motor, INPUT);
  pinMode(batt, INPUT);
  pinMode(hall, INPUT);
  pinMode(pdiode, INPUT);
  pinMode(mosfet, OUTPUT);
  pinMode(cc, OUTPUT);
  digitalWrite(mosfet, LOW);
  timeValue = EEPROM.read(timeAddress);
  distanceValue = EEPROM.read(distanceAddress);
  powerValue = EEPROM.read(powerAddress);
  root(); //set display to root menu
}
uint8_t i=0;
void loop() {
  menuFunction(); // see if button is pressed
  motorV = getmotorV(); //motor/generator output voltage
  if (motorV > 1.0 && !hasBeenOn) {
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("Warming up...");
    lcd.setCursor(0,1);
    lcd.print("Keep pedaling. ");
    lcd.setBacklight(GREEN);
    digitalWrite(cc, HIGH); //press start on charge controller
    lcd.setBacklight(YELLOW);
    delay(3500); //press start for 3.5 seconds
    cycle = 0;
    t0 = float(millis());
    digitalWrite(cc, LOW); //stop pressing start
    //battery should now be charging
    lcd.clear();
    lcd.setCursor(0,0);
    hasBeenOn = true;
    lcd.print("Charging battery");
    lcd.setBacklight(RED);
    lcd.setCursor(3, 1);
    timeOn = millis();
    //time of how long person has been pedaling
    lcd.print((millis()-timeOn)/1000);
    delay(wait);
    isHome = false;
  }
  else if(motorV > 1.0 && hasBeenOn){
    secs = int((millis()-timeOn)/1000);
    mins = int(secs/60);
    secs = int(secs%60); //this could also be written as a separate function
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print(mins);
    lcd.setCursor(2, 0);
    // print the number of seconds since start biking
    lcd.print(":");
    lcd.setCursor(3, 0);
    lcd.print(secs);
    lcd.setCursor(9, 1);
    lcd.print(rpm);
    lcd.setCursor(13,1);
    lcd.print("RPM");
    isHome = false;
    getCurrent();
    getkmh();
    delay(wait);
  }
  else{
    hasBeenOn = false;
    if (timeValue > (millis()-timeOn/1000/60)){
      timeValue = int(millis()-timeOn/1000/60);
      EEPROM.write(timeAddress, timeValue);
    }
    if(!isHome){
      root();
    }
  }
  inverterControl();
  getRpm();
}

void inverterControl() {
  battV = getbattV();
  timecheck = millis() - timeOn;
  if (hasBeenOn && (battV > 10.0) && (timecheck > 5000) && !mosfetOn) {
    digitalWrite(mosfet, HIGH);
    mosfetOn = true;
  } 
  else if ((battV <= 10.0)) {
    digitalWrite(mosfet, LOW);
    mosfetOn = false;

  } 
  else if(timecheck <5000) {
    digitalWrite(mosfet, LOW);
    mosfetOn = false;
    cycle = 0; //this is a safety since arduino can't run multiple threads
    t0 = float(millis());
    menuFunction();
  }
}
float getmotorV(){
  return (float( analogRead(motor) ) / afactor * 5.0);
}

float getbattV(){
  return (float(analogRead(batt)) / afactor * 14.0);
}

void getRpm() {
  // consider an if else/ boolean that makes sure increasing cycle only when biking
  if (t0 == 0.0) {
    t0 = float (millis());
  }
  photodiode = analogRead(pdiode);
  if (((photodiode != 0) && (analogRead(pdiode) == 0)) || ((photodiode == 0) && (analogRead(pdiode) != 0))) {
    cycle++;
    t1 = float(millis());
    if (cycle > numCycle) {
      rpm = (float(cycle)) / (t1 - t0)* 1000.0 * 60.0;
      cycle = 0;
      t0 = float(millis());
    }
  }
}

void getkmh() {
  kmh = rpm*60.0*revolution;
  lcd.setCursor(0, 1);
  lcd.print(kmh);
  lcd.setCursor(2,1);
  lcd.print("km/h   ");
} 

void getCurrent(){
  current = (float(analogRead(hall))-514.5)/26.5;
  lcd.setCursor(6, 0);
  lcd.print(int (current*13));
  lcd.setCursor(8,0);
  lcd.print("W");
  Wh = Wh + float(wait)/3600.0*current*13.0;
  lcd.setCursor(10,0);
  lcd.print(Wh);
  lcd.setCursor(13,0);
  lcd.print("Wh");
}


void menuFunction() {
  delay(200);
  uint8_t buttons = lcd.readButtons();
  if (buttons) {
    if (buttons & BUTTON_UP) {
      scrollUp(ptr);
    }
    if (buttons & BUTTON_DOWN) {
      if(ptr >0){
        scrollDown(ptr);
      }
    }
    if (buttons & BUTTON_LEFT) {          
      if(ptr >=1 && ptr <=4){
        root();
      }
      else if(ptr >= 5){
        menu();
      }
    }
    if (buttons & BUTTON_RIGHT) {
      scrollRight();
    }
  }
}


void menu() {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("MENU  (scroll V)");
  lcd.setCursor(0, 1);
  lcd.print("Top times");
  ptr = 1;
}

void root() {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Bike to Charge!");
  lcd.setCursor(0, 1);
  lcd.print("Menu (Right >)");
  ptr = 0;
  isHome = true;
}

void scrollRight() {
  Serial.println(ptr);
  if(ptr == 0){
    menu();
  }
  else if(ptr == 1){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Top time");
    lcd.setCursor(0, 1);
    lcd.print(timeValue); // RECALL NUMBER!!! TOP TIME
    lcd.setCursor(13,1);
    lcd.print("min");
    ptr = 5;
  }   
  else if(ptr == 2){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Total distance");
    lcd.setCursor(0, 1);
    lcd.print(distanceValue); // RECALL NUMBER!!! TOTAL DISTANCE
    lcd.setCursor(14,1);
    lcd.print("mi");
    ptr = 6;
  }  
  else if(ptr == 3){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Total energy");
    lcd.setCursor(0, 1);
    lcd.print(powerValue); // RECALL NUMBER!!! TOTAL WATTHOURS
    lcd.setCursor(15,1);
    lcd.print("J");
    ptr = 7;
  }
  else if(ptr == 4){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Scroll down to  ");
    lcd.setCursor(0, 1);
    lcd.print("read more!!! (V)"); // RECALL NUMBER!!! TOTAL WATTHOURS
    ptr = 8;
  }
}
void scrollDown(int i){
  Serial.println(i);
  if (i == 1){
    lcd.setCursor(0, 1);
    lcd.print("Total distance  ");
    ptr = 2;
  }
  else if (i == 2){
    lcd.setCursor(0, 1);
    lcd.print("Total energy    ");
    ptr = 3;
  }
  else if (i == 3){
    lcd.setCursor(0, 1);
    lcd.print("About!          ");
    ptr = 4;
  }
  else if (i == 8){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Electronics bike");
    lcd.setCursor(0, 1);
    lcd.print("worked on by:   ");
    ptr = 9;
  }
  else if (i == 9){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("A. McKay     '13");
    lcd.setCursor(0, 1);
    lcd.print("J. Wong      '15");
    ptr = 10;
  }
  else if (i == 10){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("A.Karapetrova'15");
    lcd.setCursor(0, 1);
    lcd.print("S. Walecka   '15");
    ptr = 11;
  }
  else if (i == 11){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("S. Li        '17");
    lcd.setCursor(0, 1);
    lcd.print("N. Sandford  '17");
    ptr = 12;
  }
  else if (i == 12){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("For His Majesty ");
    lcd.setCursor(0, 1);
    lcd.print("Dwight Whitaker ");
    ptr = 13;
  }
  else if (i == 13){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Phys 128        ");
    lcd.setCursor(0, 1);
    lcd.print("Pomona College  ");
    ptr = 14;
  }
  else if (i == 14){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Paid for by the ");
    lcd.setCursor(0, 1);
    lcd.print("SIO and Dept of ");
    ptr = 15;
  }
  else if (i == 15){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Physics and     ");
    lcd.setCursor(0, 1);
    lcd.print("Astronomy.      ");
    ptr = 16;
  }
}
void scrollUp(int i){
  if (i ==2){
    menu();
  }
  if (i>2){
    scrollDown(i-2);
  }
}

